-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THFillLevelMoistureDisplay = {}
local THFillLevelMoistureDisplay_mt = THUtils.createClass(THFillLevelMoistureDisplay)
local debugFlagId = THDebugUtil.createDebugFlag("THHud")
THFillLevelMoistureDisplay.debugFlagId = debugFlagId
THFillLevelMoistureDisplay.MOISTURE_BAR_PADDING = {
    [true] = { 2.5, 2 }
}
THFillLevelMoistureDisplay.COLOR = {
    MOISTURE_BAR = {
        LOW     = { 1.0, 0.0, 0.0, 1 },
        MID     = { 1.0, 1.0, 0.0, 1 },
        HIGH    = { 0.0, 1.0, 0.0, 1 }
    },
    MOISTURE_INDICATOR = {
        LOW     = { 1.0, 0.0, 0.0, 0.8 },
        MID     = { 1.0, 1.0, 0.0, 0.8 },
        HIGH    = { 0.0, 1.0, 0.0, 0.8 }
    }
}
local function getCustomData(self)
    return g_thMoistureSystem:getDataTable(self)
end
THFillLevelMoistureDisplay.getCustomData = getCustomData
function THFillLevelMoistureDisplay.newData(parent, owner, vehicleInfoDisplay, customMt)
    customMt = customMt or THFillLevelMoistureDisplay_mt
    local self = setmetatable({}, customMt)
    self.owner = owner
    self.parent = parent
    self.isClient = owner.isClient
    self.isServer = owner.isServer
    self.vehicleInfoDisplay = vehicleInfoDisplay
    self.moistureHudBox = self.vehicleInfoDisplay:createBox(THExtendedHudBox)
    self.fillTypeMoistureBuffer = {}
    self.moistureBufferMapping = {}
    self.fillTypeMoistureBars = {}
    return self
end
function THFillLevelMoistureDisplay:getMoistureIndicatorColor(fillType)
    local owner = self.owner
    local fillTypeMoistureData = owner:getFillTypeMoistureData(fillType)
    local r, g, b, a = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_INDICATOR.MID)
    if fillTypeMoistureData ~= nil then
        local fillTypeIndex = fillTypeMoistureData.fillType.index
        local bufferIndex = self.moistureBufferMapping[fillTypeIndex]
        if bufferIndex ~= nil then
            local bufferData = self.fillTypeMoistureBuffer[bufferIndex]
            local moistureFactors = fillTypeMoistureData.factors
            local minPriceFactor = moistureFactors.minPrice
            local maxPriceFactor = moistureFactors.maxPrice
            local curPriceFactor = bufferData.priceFactor or 1
            local minYieldFactor = moistureFactors.minYield
            local maxYieldFactor = moistureFactors.maxYield
            local curYieldFactor = bufferData.yieldFactor or 1
            curPriceFactor = THUtils.getFactorInRange(minPriceFactor, maxPriceFactor, curPriceFactor)
            curYieldFactor = THUtils.getFactorInRange(minYieldFactor, maxYieldFactor, curYieldFactor)
            local colorFactor = math.min(curPriceFactor, curYieldFactor)
            local minR, minG, minB, minA = r, g, b, a
            local maxR, maxG, maxB, maxA = r, g, b, a
            if colorFactor <= 0.5 then
                colorFactor = MathUtil.clamp(colorFactor * 2, 0, 1)
                minR, minG, minB, minA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_INDICATOR.LOW)
                maxR, maxG, maxB, maxA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_INDICATOR.MID)
            else
                colorFactor = MathUtil.clamp((colorFactor - 0.5) * 2, 0, 1)
                minR, minG, minB, minA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_INDICATOR.MID)
                maxR, maxG, maxB, maxA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_INDICATOR.HIGH)
            end
            r = THUtils.getValueInRange(minR, maxR, colorFactor)
            g = THUtils.getValueInRange(minG, maxG, colorFactor)
            b = THUtils.getValueInRange(minB, maxB, colorFactor)
            a = THUtils.getValueInRange(minA, maxA, colorFactor)
        end
    end
    return r, g, b, a
end
function THFillLevelMoistureDisplay:getMoistureBarColor(fillType)
    local owner = self.owner
    local fillTypeMoistureData = owner:getFillTypeMoistureData(fillType)
    local r, g, b, a = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_BAR.MID)
    if fillTypeMoistureData ~= nil then
        local fillTypeIndex = fillTypeMoistureData.fillType.index
        local bufferIndex = self.moistureBufferMapping[fillTypeIndex]
        if bufferIndex ~= nil then
            local bufferData = self.fillTypeMoistureBuffer[bufferIndex]
            local moistureFactors = fillTypeMoistureData.factors
            local minPriceFactor = moistureFactors.minPrice
            local maxPriceFactor = moistureFactors.maxPrice
            local curPriceFactor = bufferData.priceFactor or 1
            local minYieldFactor = moistureFactors.minYield
            local maxYieldFactor = moistureFactors.maxYield
            local curYieldFactor = bufferData.yieldFactor or 1
            curPriceFactor = THUtils.getFactorInRange(minPriceFactor, maxPriceFactor, curPriceFactor)
            curYieldFactor = THUtils.getFactorInRange(minYieldFactor, maxYieldFactor, curYieldFactor)
            local colorFactor = math.min(curPriceFactor, curYieldFactor)
            local minR, minG, minB, minA = r, g, b, a
            local maxR, maxG, maxB, maxA = r, g, b, a
            if colorFactor <= 0.5 then
                colorFactor = MathUtil.clamp(colorFactor * 2, 0, 1)
                minR, minG, minB, minA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_BAR.LOW)
                maxR, maxG, maxB, maxA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_BAR.MID)
            else
                colorFactor = MathUtil.clamp((colorFactor - 0.5) * 2, 0, 1)
                minR, minG, minB, minA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_BAR.MID)
                maxR, maxG, maxB, maxA = THUtils.unpack(THFillLevelMoistureDisplay.COLOR.MOISTURE_BAR.HIGH)
            end
            r = THUtils.getValueInRange(minR, maxR, colorFactor)
            g = THUtils.getValueInRange(minG, maxG, colorFactor)
            b = THUtils.getValueInRange(minB, maxB, colorFactor)
            a = THUtils.getValueInRange(minA, maxA, colorFactor)
        end
    end
    return r, g, b, a
end
function THFillLevelMoistureDisplay:updateFillTypeMoistureBuffer()
    local owner = self.owner
    local moistureBuffer = THUtils.clearTable(self.fillTypeMoistureBuffer)
    local bufferMapping = THUtils.clearTable(self.moistureBufferMapping)
    for _, moistureBar in pairs(self.fillTypeMoistureBars) do
        moistureBar:setVisible(false)
    end
    local function updateBufferData(pVehicle)
        local thMoistureFillUnitSpec = THMoistureFillUnit.getSpecTable(pVehicle, false)
        if thMoistureFillUnitSpec ~= nil then
            local fillUnitsArray = pVehicle:getFillUnits()
            local numFillUnits = #fillUnitsArray
            if numFillUnits > 0 then
                for fillUnitIndex = 1, numFillUnits do
                    local fillUnitMoisture, fillUnitFillLevel, fillUnitFillType = pVehicle:thAA_getFillUnitMoisture(fillUnitIndex)
                    if fillUnitMoisture ~= nil and fillUnitMoisture >= 0
                        and fillUnitFillLevel ~= nil and fillUnitFillLevel > 0
                        and fillUnitFillType ~= nil and fillUnitFillType ~= FillType.UNKNOWN
                    then
                        local fillTypeMoistureData = owner:getFillTypeMoistureData(fillUnitFillType)
                        if fillTypeMoistureData ~= nil then
                            local bufferIndex = bufferMapping[fillUnitFillType]
                            local minMoisture = fillTypeMoistureData.minPercent
                            local maxMoisture = fillTypeMoistureData.maxPercent
                            local bufferData = nil
                            if bufferIndex == nil then
                                bufferData = {
                                    fillTypeIndex = fillUnitFillType,
                                    fillLevel = fillUnitFillLevel,
                                    moisture = fillUnitMoisture
                                }
                                table.insert(moistureBuffer, bufferData)
                                bufferMapping[fillUnitFillType] = #moistureBuffer
                            else
                                bufferData = moistureBuffer[bufferIndex] -- should exist
                                local totalFillLevel = bufferData.fillLevel + fillUnitFillLevel
                                local newFillPercent = MathUtil.clamp(fillUnitFillLevel / totalFillLevel, 0, 1)
                                local oldFillPercent = 1 - newFillPercent
                                bufferData.fillLevel = totalFillLevel
                                bufferData.moisture = MathUtil.clamp((bufferData.moisture * oldFillPercent) + (fillUnitMoisture * newFillPercent), minMoisture, maxMoisture)
                            end
                            bufferData.wetFactor = THUtils.getFactorInRange(minMoisture, maxMoisture, bufferData.moisture)
                            bufferData.dryFactor = 1 - bufferData.wetFactor
                            local priceFactor, yieldFactor = owner:getFillTypeMoistureFactors(fillUnitFillType, bufferData.moisture)
                            bufferData.priceFactor = priceFactor or 1
                            bufferData.yieldFactor = yieldFactor or 1
                        end
                    end
                end
            end
        end
    end
    if self.vehicle ~= nil and self.vehicle.thAA_callChainFunc ~= nil then
        self.vehicle:thAA_callChainFunc(updateBufferData)
    end
end
function THFillLevelMoistureDisplay.hook_new(superFunc, ...)
    local function appendFunc(rDisplay, ...)
        if rDisplay ~= nil then
            THUtils.call(function()
                local thHud = THHud.getCustomData(g_thMoistureSystem.hud)
                if thHud ~= nil then
                    local customData = g_thMoistureSystem:createDataTable(rDisplay, false, THFillLevelMoistureDisplay, thHud.vehicleInfoDisplay)
                end
            end)
        end
        return rDisplay, ...
    end
    return appendFunc(superFunc(...))
end
function THFillLevelMoistureDisplay:hook_setVehicle(superFunc, vehicle, ...)
    local customData = getCustomData(self)
    local function appendFunc(...)
        if customData ~= nil then
            THUtils.call(function()
                customData.vehicle = self.vehicle
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, vehicle, ...))
end
function THFillLevelMoistureDisplay:hook_draw(superFunc, ...)
    local customData = getCustomData(self)
    local function appendFunc(...)
        if customData ~= nil then
            local owner = customData.owner
            local thFillTypeManager = owner.thFillTypeManager
            THUtils.call(function()
                local hudBox = customData.moistureHudBox
                local hudTitle = owner.i18n:getText("thUI_hudTitleMoisture")
                hudBox:clear()
                hudBox:xtSetTitle(hudTitle, KeyValueInfoHUDBox.COLOR.TEXT_HIGHLIGHT)
                local numBufferEntries = #customData.fillTypeMoistureBuffer
                if numBufferEntries > 0 then
                    for bufferIndex = numBufferEntries, 1, -1 do -- reverse order like fill bars
                        local bufferData = customData.fillTypeMoistureBuffer[bufferIndex]
                        local fillTypeIndex = bufferData.fillTypeIndex
                        local fillTypeInfo = thFillTypeManager:getFillType(fillTypeIndex)
                        local fillTypeMoisture = bufferData.moisture
                        local moistureText = THUtils.formatPercent(fillTypeMoisture, 1)
                        local indicatorColor = THUtils.pack(customData:getMoistureIndicatorColor(fillTypeIndex))
                        if indicatorColor ~= nil then
                            moistureText = string.format("%s", moistureText)
                            hudBox:xtAddLine(fillTypeInfo.title, moistureText, true, nil, true, nil, nil, indicatorColor)
                        end
                    end
                    hudBox:showNextFrame()
                end
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, ...))
end
function THFillLevelMoistureDisplay:hook_updateFillLevelFrames(superFunc, ...)
    local customData = getCustomData(self)
    local function appendFunc(...)
        if customData ~= nil then
            THUtils.call(function()
                local fillLevelBuffer = self.fillLevelBuffer
                local numBufferEntries = 0
                if fillLevelBuffer ~= nil then
                    numBufferEntries = #fillLevelBuffer
                end
                customData:updateFillTypeMoistureBuffer()
                if numBufferEntries > 0 then
                    for bufferIndex = 1, numBufferEntries do
                        local bufferInfo = fillLevelBuffer[bufferIndex]
                        local fillTypeIndex = bufferInfo.fillType
                        if fillTypeIndex ~= nil then
                            local fillTypeFrame = self.fillTypeFrames[fillTypeIndex]
                            if fillTypeFrame ~= nil and fillTypeFrame:getVisible() then
                                local fillTypeMoistureBar = customData.fillTypeMoistureBars[fillTypeIndex]
                                local moistureBufferIndex = customData.moistureBufferMapping[fillTypeIndex]
                                if fillTypeMoistureBar ~= nil and moistureBufferIndex ~= nil then
                                    local moistureBufferData = customData.fillTypeMoistureBuffer[moistureBufferIndex]
                                    local wetFactor = moistureBufferData.wetFactor
                                    local barColorR, barColorG, barColorB = customData:getMoistureBarColor(fillTypeIndex)
                                    fillTypeMoistureBar:setBarColor(barColorR, barColorG, barColorB)
                                    fillTypeMoistureBar:setValue(wetFactor)
                                    fillTypeMoistureBar:setVisible(true)
                                end
                            end
                        end
                    end
                end
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, ...))
end
function THFillLevelMoistureDisplay:hook_createFillTypeBar(superFunc, hudAtlasPath, frame, baseX, baseY, fillTypeInfo, ...)
    local customData = getCustomData(self)
    local function appendFunc(...)
        if customData ~= nil then
            THUtils.call(function()
                if frame ~= nil and fillTypeInfo ~= nil then
                    local fillTypeIndex = fillTypeInfo.index
                    local fillTypeBar = self.fillTypeLevelBars[fillTypeIndex]
                    if fillTypeBar ~= nil and fillTypeBar.overlay ~= nil then
                        local parentPosX, parentPosY = THUtils.unpack(FillLevelsDisplay.POSITION.BAR)
                        local parentWidth, parentHeight = THUtils.unpack(FillLevelsDisplay.SIZE.BAR)
                        local moistureBarWidth = parentWidth
                        local moistureBarHeight = parentHeight * 0.5
                        local moistureBarPosX = parentPosX
                        local moistureBarPosY = parentPosY - moistureBarHeight
                        moistureBarPosX, moistureBarPosY = getNormalizedScreenValues(moistureBarPosX, moistureBarPosY)
                        moistureBarWidth, moistureBarHeight = getNormalizedScreenValues(moistureBarWidth, moistureBarHeight)
                        moistureBarPosX = baseX + moistureBarPosX
                        moistureBarPosY = baseY + moistureBarPosY
                        local element = HUDRoundedBarElement.new(hudAtlasPath, moistureBarPosX, moistureBarPosY, moistureBarWidth, moistureBarHeight, true)
                        local oldUpdateComponents = element.updateComponents
                        element.updateComponents = function(pSelf, pBarOnly, ...)
                            local globalEnv = THUtils.getGlobalEnvironment()
                            local HUDRoundedBarElement = globalEnv.HUDRoundedBarElement
                            local oldHorizontalPadding = HUDRoundedBarElement.PADDING[true]
                            HUDRoundedBarElement.PADDING[true] = THFillLevelMoistureDisplay.MOISTURE_BAR_PADDING[true]
                            local function appendFunc2(...)
                                HUDRoundedBarElement.PADDING[true] = oldHorizontalPadding
                                return ...
                            end
                            return appendFunc2(oldUpdateComponents(pSelf, pBarOnly, ...))
                        end
                        frame:addChild(element)
                        customData.fillTypeMoistureBars[fillTypeIndex] = element
                    end
                end
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, hudAtlasPath, frame, baseX, baseY, fillTypeInfo, ...))
end
g_thMoistureSystem:addDelayedLoadTask(function()
    THUtils.setFunctionHook("FillLevelsDisplay", "new", THFillLevelMoistureDisplay, nil, false)
    THUtils.setFunctionHook("FillLevelsDisplay", "setVehicle", THFillLevelMoistureDisplay)
    THUtils.setFunctionHook("FillLevelsDisplay", "draw", THFillLevelMoistureDisplay)
    THUtils.setFunctionHook("FillLevelsDisplay", "updateFillLevelFrames", THFillLevelMoistureDisplay)
    THUtils.setFunctionHook("FillLevelsDisplay", "createFillTypeBar", THFillLevelMoistureDisplay)
end)