-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THInGameMoistureWeatherFrame = {}
local THInGameMoistureWeatherFrame_mt = THUtils.createClass(THInGameMoistureWeatherFrame)
local debugFlagId = THDebugUtil.createDebugFlag("THInGameMenu")
THInGameMoistureWeatherFrame.debugFlagId = debugFlagId
local function getCustomData(self)
    return g_thMoistureSystem:getDataTable(self)
end
THInGameMoistureWeatherFrame.getCustomData = getCustomData
function THInGameMoistureWeatherFrame.newData(parent, owner, customMt)
    customMt = customMt or THInGameMoistureWeatherFrame_mt
    local self = setmetatable({}, customMt)
    self.owner = owner
    self.parent = parent
    self.isClient = owner.isClient
    self.isServer = owner.isServer
    local nowTemperature = parent.nowTemperature
    local nowContainer = nowTemperature.parent
    self.nowHumidity = nowTemperature:clone(nowContainer)
    self.nowHumidity:applyProfile("thInGameMenuWeatherNowHumidity")
    self.nowHumidity:setText("0%")
    local humidityTitle = owner.i18n:getText("thMenuTitle_humidity")
    self.nowHumidityTitle = nowTemperature:clone(nowContainer)
    self.nowHumidityTitle:applyProfile("thInGameMenuWeatherNowHumidityTitle")
    self.nowHumidityTitle:setText(humidityTitle)
    return self
end
function THInGameMoistureWeatherFrame:hook_onFrameOpen(superFunc, ...)
    local customData = getCustomData(self)
    THUtils.call(function()
        if customData == nil then
            customData = g_thMoistureSystem:createDataTable(self, false, THInGameMoistureWeatherFrame)
        end
    end)
    return superFunc(self, ...)
end
function THInGameMoistureWeatherFrame:hook_updateTodayView(superFunc, ...)
    local customData = getCustomData(self)
    local function appendFunc(...)
        if customData ~= nil then
            local owner = customData.owner
            THUtils.call(function()
                local curHumidity = owner:getCurrentHumidityValues()
                local curHumidityText = THUtils.formatPercent(curHumidity, 1)
                customData.nowHumidity:setText(curHumidityText)
                customData.nowHumidity:setVisible(true)
                customData.nowHumidityTitle:setVisible(true)
            end)
        end
    end
    return appendFunc(superFunc(self, ...))
end
g_thMoistureSystem:addDelayedLoadTask(function()
    THUtils.setFunctionHook("InGameMenuWeatherFrame", "onFrameOpen", THInGameMoistureWeatherFrame)
    THUtils.setFunctionHook("InGameMenuWeatherFrame", "updateTodayView", THInGameMoistureWeatherFrame)
end)