-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THPlayerMoistureHud = {}
local THPlayerMoistureHud_mt = THUtils.createClass(THPlayerMoistureHud)
local debugFlagId = THDebugUtil.createDebugFlag("THHud")
THPlayerMoistureHud.debugFlagId = debugFlagId
local function getCustomData(self)
    return g_thMoistureSystem:getDataTable(self)
end
THPlayerMoistureHud.getCustomData = getCustomData
function THPlayerMoistureHud.newData(parent, owner, customMt)
    customMt = customMt or THPlayerMoistureHud_mt
    local self = setmetatable({}, customMt)
    self.owner = owner
    self.parent = parent
    self.hasPFExtension = false
    local thPFManager = owner.thPFManager
    local pfMain = thPFManager.parent
    if pfMain ~= nil and pfMain.fieldInfoDisplayExtension ~= nil then
        local pfExtensionData = owner:getDataTable(pfMain.fieldInfoDisplayExtension)
        if pfExtensionData == nil then
            local pfModEnv = thPFManager.pfModEnvironment
            pfExtensionData = owner:createDataTable(pfMain.fieldInfoDisplayExtension, false, false)
            THUtils.setFunctionHook(pfModEnv.FieldInfoDisplayExtension, "infoBoxAddYieldData", nil, THPlayerMoistureHud.hook_pfInfoBoxAddYieldData, true, self)
        end
        self.hasPFExtension = true
    end
    return self
end
function THPlayerMoistureHud:addFruitTypeMoistureToHudBox(box, fruitTypeIndex, growthState)
    local owner = self.owner
    local mission = owner.mission
    local player = mission.player
    local px, _, pz = player:getPositionData()
    local farmlandManager = owner.farmlandManager
    local thPFManager = owner.thPFManager
    local farmlandId = farmlandManager:getFarmlandIdAtWorldPosition(px, pz)
    local soilTypeIndex = thPFManager:getSoilTypeIndexAtWorldPos(px, pz, true) -- force
    local fruitTypeMoisture = owner:getFruitTypeMoisture(fruitTypeIndex, growthState, nil, farmlandId, soilTypeIndex)
    if fruitTypeMoisture ~= nil and fruitTypeMoisture >= 0 then
        local moistureText = THUtils.formatPercent(fruitTypeMoisture, 1)
        local moistureTitle = owner.i18n:getText("thUI_hudTitleMoisture")
        box:addLine(moistureTitle, moistureText)
    end
end
function THPlayerMoistureHud.hook_new(superFunc, ...)
    local function appendFunc(rUpdater, ...)
        if rUpdater ~= nil then
            THUtils.call(function()
                local customData = g_thMoistureSystem:createDataTable(rUpdater, false, THPlayerMoistureHud)
            end)
        end
        return rUpdater, ...
    end
    return appendFunc(superFunc(...))
end
function THPlayerMoistureHud:hook_fieldAddFruit(superFunc, data, box, ...)
    local customData = getCustomData(self)
    local fruitTypeIndex = FruitType.UNKNOWN
    local fruitGrowthState = -1
    if customData ~= nil and data ~= nil and box ~= nil
        and not customData.hasPFExtension
    then
        THUtils.call(function()
            fruitTypeIndex = data.fruitTypeMax or FruitType.UNKNOWN
            fruitGrowthState = data.fruitStateMax or -1
        end)
    end
    local function appendFunc(...)
        if customData ~= nil
            and fruitGrowthState >= 0 and fruitTypeIndex ~= FruitType.UNKNOWN -- implies other checks
        then
            THUtils.call(function()
                customData:addFruitTypeMoistureToHudBox(box, fruitTypeIndex, fruitGrowthState)
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, data, box, ...))
end
function THPlayerMoistureHud:hook_pfInfoBoxAddYieldData(superFunc, customData, data, box, ...)
    local fruitTypeIndex = FruitType.UNKNOWN
    local fruitGrowthState = -1
    if customData ~= nil then
        THUtils.call(function()
            local owner = customData.owner
            local pfExtensionData = owner:getDataTable(self)
            if pfExtensionData ~= nil and data ~= nil and box ~= nil then
                fruitTypeIndex = data.fruitTypeMax or FruitType.UNKNOWN
                fruitGrowthState = data.fruitStateMax or -1
            end
        end)
    end
    local function appendFunc(...)
        if customData ~= nil
            and fruitGrowthState >= 0 and fruitTypeIndex ~= FruitType.UNKNOWN -- implies other checks
        then
            THUtils.call(function()
                customData:addFruitTypeMoistureToHudBox(box, fruitTypeIndex, fruitGrowthState)
            end)
        end
        return ...
    end
    return appendFunc(superFunc(self, data, box, ...))
end
g_thMoistureSystem:addDelayedLoadTask(function()
    THUtils.setFunctionHook("PlayerHUDUpdater", "new", THPlayerMoistureHud, nil, false)
    THUtils.setFunctionHook("PlayerHUDUpdater", "fieldAddFruit", THPlayerMoistureHud)
end)