-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

local thModName = g_thMoistureSystem.modName
THMoistureCombine = {}
local debugFlagId = THDebugUtil.createDebugFlag("THCombine")
THMoistureCombine.debugFlagId = debugFlagId
THMoistureCombine.MOD_NAME = thModName
THMoistureCombine.SPEC_NAME = "moistureCombine"
local function getSpecTable(self, verbose)
    return THUtils.getSpecTable(self, thModName, THMoistureCombine.SPEC_NAME, verbose)
end
THMoistureCombine.getSpecTable = getSpecTable
function THMoistureCombine.initSpecialization()
end
function THMoistureCombine.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(THMoistureFillUnit, specializations)
        and SpecializationUtil.hasSpecialization(THCombine, specializations)
end
function THMoistureCombine.registerEvents(vehicleType)
end
function THMoistureCombine.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onPreLoad", THMoistureCombine)
end
function THMoistureCombine.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "thAA_getCombineInputMoisture", THMoistureCombine.thAA_getCombineInputMoisture)
end
function THMoistureCombine.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "addCutterArea", THMoistureCombine.addCutterArea)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "addFillUnitFillLevel", THMoistureCombine.addFillUnitFillLevel)
end
function THMoistureCombine:onPreLoad(savegame)
    local specTable = getSpecTable(self)
    specTable.currentInputMoisture = -1
    specTable.lastCutterParameters = {}
end
function THMoistureCombine:thAA_getCombineInputMoisture()
    local specTable = getSpecTable(self)
    return specTable.currentInputMoisture
end
function THMoistureCombine:addCutterArea(superFunc, changedArea, realArea, inputFruitType, outputFillType, strawRatio, chopperType, farmId, cutterLoad, ...)
    local specTable = getSpecTable(self)
    specTable.currentInputMoisture = -1
    THUtils.call(function()
        if inputFruitType ~= nil and inputFruitType ~= FruitType.UNKNOWN
            and outputFillType ~= nil and outputFillType ~= FillType.UNKNOWN
        then
            local cutterVehicle = self:thAA_getCombineLastValidCutter()
            local thCutterSpec = THCutter.getSpecTable(cutterVehicle, false)
            local cutterParams = THUtils.createSubTable(specTable.lastCutterParameters, inputFruitType, true)
            local fillTypeMoisture = -1
            if cutterVehicle ~= nil and thCutterSpec ~= nil then
                local cutterSpec = cutterVehicle.spec_cutter
                local growthState = cutterSpec.currentGrowthState or -1
                local farmlandId = thCutterSpec.lastTouchedFarmland
                local soilTypeIndex = thCutterSpec.lastTouchedSoilType
                if growthState >= 0 then
                    fillTypeMoisture = g_thMoistureSystem:getFillTypeMoisture(inputFruitType, outputFillType, growthState, nil, farmlandId, soilTypeIndex)
                    if fillTypeMoisture ~= nil and fillTypeMoisture >= 0 then
                        specTable.currentInputMoisture = fillTypeMoisture
                    end
                end
                cutterParams.growthState = growthState
                cutterParams.farmlandId = farmlandId
                cutterParams.soilTypeIndex = soilTypeIndex
            end
        end
    end)
    local function appendFunc(...)
        specTable.currentInputMoisture = -1
        return ...
    end
    return appendFunc(superFunc(self, changedArea, realArea, inputFruitType, outputFillType, strawRatio, chopperType, farmId, cutterLoad, ...))
end
function THMoistureCombine:addFillUnitFillLevel(superFunc, farmId, fillUnitIndex, deltaFill, fillTypeIndex, ...)
    local combineSpec = self.spec_combine
    if fillUnitIndex ~= nil
        and deltaFill ~= nil and deltaFill > 0
        and fillTypeIndex ~= nil and fillTypeIndex ~= FillType.UNKNOWN
        and (fillUnitIndex == combineSpec.bufferFillUnitIndex or fillUnitIndex == combineSpec.fillUnitIndex)
    then
        THUtils.call(function()
            local fillTypeMoisture = self:thAA_getCombineInputMoisture()
            if fillTypeMoisture >= 0 then
                self:thAA_setFillUnitInputMoisture(fillUnitIndex, fillTypeIndex, fillTypeMoisture)
            end
        end)
    end
    return superFunc(self, farmId, fillUnitIndex, deltaFill, fillTypeIndex, ...)
end